import ClearIcon from "@mui/icons-material/Clear";
import DeleteIcon from "@mui/icons-material/Delete";
import EditIcon from "@mui/icons-material/Edit";
import SaveIcon from "@mui/icons-material/Save";
import { Button, Grid, Stack } from "@mui/material";
import React from "react";
import { useNavigate, useParams } from "react-router-dom";
import { Member, MemberApi } from "../../api/MemberApi";
import { ServerApi } from "../../api/ServerApi";
import { useAlert } from "../../hooks/context_providers/AlertDialogProvider";
import { useConfirm } from "../../hooks/context_providers/ConfirmDialogProvider";
import { useSnackbar } from "../../hooks/context_providers/SnackbarProvider";
import { AsyncWidget } from "../../widgets/AsyncWidget";
import { useFamily } from "../../widgets/BaseFamilyRoute";
import { ConfirmLeaveWithoutSaveDialog } from "../../widgets/ConfirmLeaveWithoutSaveDialog";
import { FamilyPageTitle } from "../../widgets/FamilyPageTitle";
import { PropEdit } from "../../widgets/PropEdit";
import { PropertiesBox } from "../../widgets/PropertiesBox";
import { SexSelection } from "../../widgets/SexSelection";

/**
 * Create a new member route
 */
export function FamilyCreateMemberRoute(): React.ReactElement {
  const alert = useAlert();
  const snackbar = useSnackbar();

  const [shouldQuit, setShouldQuit] = React.useState(false);
  const n = useNavigate();
  const family = useFamily();

  const create = async (m: Member) => {
    try {
      const r = await MemberApi.Create(m);

      await family.reloadMembersList();

      setShouldQuit(true);
      n(family.family.URL(`member/${r.id}`));
      snackbar(`La fiche pour ${r.fullName} a été créée avec succès !`);
    } catch (e) {
      console.error(e);
      alert("Echec de la création de la personne !");
    }
  };

  const cancel = () => {
    setShouldQuit(true);
    n(family.family.URL("members"));
  };

  return (
    <MemberPage
      member={Member.New(family.family.family_id)}
      creating={true}
      editing={true}
      onCancel={cancel}
      onSave={create}
      shouldAllowLeaving={shouldQuit}
    />
  );
}

/**
 * Get existing member route
 */
export function FamilyMemberRoute(): React.ReactElement {
  const n = useNavigate();
  const alert = useAlert();
  const confirm = useConfirm();
  const snackbar = useSnackbar();

  const family = useFamily();
  const { memberId } = useParams();

  const [member, setMember] = React.useState<Member>();
  const load = async () => {
    setMember(await MemberApi.GetSingle(family.familyId, Number(memberId)));
  };

  const deleteMember = async () => {
    try {
      if (
        !(await confirm(
          "Voulez-vous vraiment supprimer cette fiche membre ? L'opération n'est pas réversible !"
        ))
      )
        return;

      await MemberApi.Delete(member!);

      snackbar("La fiche de membre a été supprimée avec succès !");
      n(family.family.URL("members"));

      await family.reloadMembersList();
    } catch (e) {
      console.error(e);
      alert("Échec de la suppression du membre !");
    }
  };

  return (
    <AsyncWidget
      loadKey={memberId}
      load={load}
      errMsg="Echec du chargement des informations du membre"
      build={() => (
        <MemberPage
          member={member!}
          creating={false}
          editing={false}
          onRequestDelete={deleteMember}
          onRequestEdit={() =>
            n(family.family.URL(`member/${member!.id}/edit`))
          }
        />
      )}
    />
  );
}

/**
 * Edit existing member route
 */
export function FamilyEditMemberRoute(): React.ReactElement {
  const n = useNavigate();
  const alert = useAlert();
  const snackbar = useSnackbar();

  const [shouldQuit, setShouldQuit] = React.useState(false);

  const family = useFamily();
  const { memberId } = useParams();

  const [member, setMember] = React.useState<Member>();
  const load = async () => {
    setMember(await MemberApi.GetSingle(family.familyId, Number(memberId)));
  };

  const cancel = () => {
    setShouldQuit(true);
    n(family.family.URL(`member/${member!.id}`));
  };

  const save = async (m: Member) => {
    try {
      await MemberApi.Update(m);

      snackbar("Les informations du membre ont été mises à jour avec succès !");

      await family.reloadMembersList();

      setShouldQuit(true);
      n(family.family.URL(`member/${member!.id}`));
    } catch (e) {
      console.error(e);
      alert("Échec de la mise à jour des informations du membre !");
    }
  };

  return (
    <AsyncWidget
      loadKey={memberId}
      load={load}
      errMsg="Échec du chargement des informations du membre"
      build={() => (
        <MemberPage
          member={member!}
          creating={false}
          editing={true}
          onCancel={cancel}
          onSave={save}
          shouldAllowLeaving={shouldQuit}
        />
      )}
    />
  );
}

export function MemberPage(p: {
  member: Member;
  editing: boolean;
  creating: boolean;
  shouldAllowLeaving?: boolean;
  onCancel?: () => void;
  onSave?: (m: Member) => void;
  onRequestEdit?: () => void;
  onRequestDelete?: () => void;
}): React.ReactElement {
  const confirm = useConfirm();

  const [changed, setChanged] = React.useState(false);
  const [member, setMember] = React.useState(structuredClone(p.member));

  const updatedMember = () => {
    setChanged(true);
    setMember(structuredClone(member));
  };

  const save = () => {
    p.onSave!(member);
  };

  const cancel = async () => {
    if (
      changed &&
      !(await confirm(
        "Voulez-vous vraiment retirer les modifications apportées ? Celles-ci seront perdues !"
      ))
    )
      return;

    p.onCancel!();
  };

  return (
    <div style={{ maxWidth: "2000px", margin: "auto" }}>
      <ConfirmLeaveWithoutSaveDialog
        shouldBlock={changed && p.shouldAllowLeaving !== true}
      />
      <div
        style={{
          display: "flex",
          justifyContent: "space-between",
          alignItems: "center",
        }}
      >
        <FamilyPageTitle
          title={
            (p.editing
              ? p.creating
                ? "Création"
                : "Édition"
              : "Visualisation") + " d'une fiche de membre"
          }
        />
        <Stack direction="row" spacing={1}>
          {/* Edit button */}
          {p.onRequestEdit && (
            <Button
              variant="outlined"
              startIcon={<EditIcon />}
              onClick={p.onRequestEdit}
              size="large"
            >
              Editer
            </Button>
          )}

          {/* Delete button */}
          {p.onRequestDelete && (
            <Button
              variant="outlined"
              startIcon={<DeleteIcon />}
              onClick={p.onRequestDelete}
              size="large"
              color="error"
            >
              Supprimer
            </Button>
          )}

          {/* Save button */}
          {p.editing && changed && (
            <Button
              variant={"contained"}
              startIcon={<SaveIcon />}
              onClick={save}
              size="large"
            >
              Enregistrer
            </Button>
          )}

          {/* Cancel button */}
          {p.editing && (
            <Button
              variant="outlined"
              startIcon={<ClearIcon />}
              onClick={cancel}
              size="small"
            >
              Annuler les modifications
            </Button>
          )}
        </Stack>
      </div>
      <Grid container spacing={2}>
        <Grid item sm={12} md={6}>
          <PropertiesBox title="Informations générales">
            {/* Sex */}
            <SexSelection
              readonly={!p.editing}
              current={member.sex}
              onChange={(v) => {
                member.sex = v;
                updatedMember();
              }}
            />

            {/* First name */}
            <PropEdit
              label="Prénom"
              editable={p.editing}
              value={member.first_name}
              onValueChange={(v) => {
                member.first_name = v;
                updatedMember();
              }}
              size={ServerApi.Config.constraints.member_first_name}
            />

            {/* Last name */}
            <PropEdit
              label="Nom"
              editable={p.editing}
              value={member.last_name}
              onValueChange={(v) => {
                member.last_name = v;
                updatedMember();
              }}
              size={ServerApi.Config.constraints.member_last_name}
            />
            <PropEdit
              label="Nom de naissance"
              editable={p.editing}
              value={member.birth_last_name}
              onValueChange={(v) => {
                member.birth_last_name = v;
                updatedMember();
              }}
              size={ServerApi.Config.constraints.member_birth_last_name}
            />
          </PropertiesBox>
        </Grid>
        <Grid item sm={12} md={6}>
          <PropertiesBox title="Contact"></PropertiesBox>
        </Grid>
        <Grid item sm={12} md={6}>
          <PropertiesBox title="Photo"></PropertiesBox>
        </Grid>
        <Grid item sm={12} md={6}>
          <PropertiesBox title="Biographie"></PropertiesBox>
        </Grid>
        <Grid item sm={12} md={6}>
          <PropertiesBox title="Époux / Épouse">TODO</PropertiesBox>
        </Grid>
        <Grid item sm={12} md={6}>
          <PropertiesBox title="Enfants">TODO</PropertiesBox>
        </Grid>
      </Grid>
    </div>
  );
}
